<?php

namespace Drupal\totext\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Psr7\Utils;
use phpDocumentor\Reflection\PseudoTypes\StringValue;
use Psr\Http\Message\ResponseInterface;

class TestForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'totext_test_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    // Retrieve the credit count.
    $credit_count = totext_get_credit_count();

    if ($credit_count) {
      $form['totext_count'] = [
        '#markup' => '<p>' . \Drupal::translation()->formatPlural($credit_count, 'You have <strong>1 credit</strong> remaining', 'You have <strong>@count credits</strong> remaining') . '</p>',
      ];
    }

    $form['destination'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Destination(s)'),
      '#description' => $this->t('A comma separated list of phone numbers to SMS.'),
    ];

    $form['message'] = [
      '#type' => 'textarea',
      '#required' => TRUE,
      '#title' => $this->t('Message'),
      '#description' => $this->t('The message to send.'),
    ];

    $form['test_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Test mode'),
      '#description' => $this->t("Make the API call in 'test mode' - validate etc, but don't actually send anything"),
      '#default_value' => FALSE,
    ];

    $form['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show full HTTP request/response (useful for debugging)'),
      '#default_value' => FALSE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send message'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the form values
    $message = $form_state->getValue('message');
    $destination = $form_state->getValue('destination');
    $test_mode = $form_state->getValue('test_mode');
    $debug = $form_state->getValue('debug');

    // Call the totext_send_message() function.
    $start_time = microtime(true);
    [$message_sent, $response] = totext_send_message($destination, $message, $test_mode);
    $end_time = microtime(true);
    $elapsed_time = $end_time - $start_time;
    if ($message_sent) {
      $this->messenger()->addStatus($this->t('Your message was successfully sent.'));
    } else {
      $responseBody = (string) $response->getBody();
      $this->messenger()->addError($this->t('Your message failed to send. API response: @response', ['@response' => strip_tags($responseBody)]));
    }


    if ($debug) {
      // Formatting the elapsed time.
      $elapsed_time_formatted = number_format($elapsed_time, 1);
      // Displaying the response information.
      $response_formatted = $this->_totext_format_http_response($response);
      $this->messenger()->addStatus(t('Response in @elapsed_time_formatted: <code><pre>@response_formatted</pre></code>', [
        '@elapsed_time_formatted' => $elapsed_time_formatted,
        '@response_formatted' => $response_formatted,
      ]));
    }

    // Keep the submitted values.
    $form_state->setRebuild(TRUE);
  }

  /**
   * Format a response object as a string.
   *
   * @param \Psr\Http\Message\ResponseInterface $response
   *   The response object.
   *
   * @return string
   *   The formatted response.
   */
  protected function _totext_format_http_response(ResponseInterface $response): string {
    $output_lines = [];
    // Add the status line.
    $output_lines[] = 'HTTP/' . $response->getProtocolVersion() . ' ' . $response->getStatusCode();

    // Add the headers.
    foreach ($response->getHeaders() as $key => $values) {
      foreach ($values as $value) {
        $output_lines[] = mb_convert_case($key, MB_CASE_TITLE) . ': ' . $value;
      }
    }

    // Add a blank line after the headers.
    $output_lines[] = '';

    // Now add the body.
    $output_lines[] = (string) $response->getBody();

    return implode("\n", $output_lines);
  }
}


